import { RangeSet } from "@codemirror/state";
function rangeSetsEqual(a, b) {
  return RangeSet.eq([b], [a]);
}
function mapRangeSetToArray(rangeSet, callbackfn) {
  const valueCount = rangeSet.size;
  const values = new Array(valueCount);
  if (valueCount > 0) {
    let valueIndex = 0;
    for (const rangeCursor = rangeSet.iter(); rangeCursor.value !== null; rangeCursor.next()) {
      values[valueIndex++] = callbackfn(rangeCursor.value, rangeCursor.from, rangeCursor.to);
    }
  }
  return values;
}
function reduceRangeSet(rangeSet, callbackfn, initialValue) {
  const rangeCursor = rangeSet.iter();
  if (!rangeCursor.value) {
    if (initialValue == null) {
      throw new TypeError("Reduce of empty RangeSet with no initial value");
    }
    return initialValue;
  }
  let accumulator;
  if (initialValue == null) {
    accumulator = rangeCursor.value;
    rangeCursor.next();
  } else {
    accumulator = initialValue;
  }
  for (; rangeCursor.value !== null; rangeCursor.next()) {
    accumulator = callbackfn(accumulator, rangeCursor.value, rangeCursor.from, rangeCursor.to);
  }
  return accumulator;
}
const SetContainerUtils = {
  add: (container, value) => {
    const set = container.extract();
    if (set.has(value)) {
      return container;
    }
    set.add(value);
    return createSetContainer(set);
  },
  delete: (container, value) => {
    const set = container.extract();
    if (!set.has(value)) {
      return container;
    }
    set.delete(value);
    return createSetContainer(set);
  },
  clear: (container) => {
    const set = container.extract();
    if (!set.size) {
      return container;
    }
    set.clear();
    return createSetContainer(set);
  },
  fp: {
    add: (value) => (container) => {
      return SetContainerUtils.add(container, value);
    },
    delete: (value) => (container) => {
      return SetContainerUtils.delete(container, value);
    }
  }
};
function createSetContainer(set = /* @__PURE__ */ new Set()) {
  const container = {
    add: (value) => SetContainerUtils.add(container, value),
    addMany: (values) => values.reduce(SetContainerUtils.add, container),
    delete: (value) => SetContainerUtils.delete(container, value),
    deleteMany: (values) => values.reduce(SetContainerUtils.delete, container),
    clear: () => SetContainerUtils.clear(container),
    extract: () => set
  };
  return container;
}
function mapStateEffectValue(effect, fn) {
  return fn(effect.value);
}
export {
  SetContainerUtils,
  createSetContainer,
  mapRangeSetToArray,
  mapStateEffectValue,
  rangeSetsEqual,
  reduceRangeSet
};
